/************************************************************************
		Company     : chiplogic 
		File        : Initiator address control
		Created on  : 14th june 1999
		Created by  : D.A.Naga raju
		modified on : th june 1999  
		Description : This generates control signals for different blocks
*************************************************************************/
`include "itr_header.v"

module itr_addr_ctl (
						I_CLK,
						I_RST_N,
						I_GNT_N,
						I_FRAME_N,
						I_IRDY_N,
						I_itr_end_xfer,
						I_latched_addr_lo,
						I_latched_addr_hi,
						I_val_addr_lo,
						I_val_addr_hi,
						I_latched_cbe_hi,
						I_latched_cbe_lo,
						I_val_req32,
						I_val_req64,
						I_dac,
						
						O_first_xfer_cycle,
						O_second_xfer_cycle,
						O_do_disconnect,
						O_addr_par,
						O_addr_par64,
						O_latched_end_xfer
					);


input		I_CLK;
input		I_IRDY_N;
input		I_FRAME_N;
input		I_RST_N;
input		I_GNT_N;
input		I_itr_end_xfer;
input[`DATA_HALF_WIDTH-1:0]		I_latched_addr_lo;
input[`DATA_HALF_WIDTH-1:0]		I_latched_addr_hi;
input		I_val_addr_lo;
input		I_val_addr_hi;
input [`CBE_HALF_WIDTH-1:0]	I_latched_cbe_lo;
input [`CBE_HALF_WIDTH-1:0]	I_latched_cbe_hi;
input		I_val_req32;
input		I_val_req64;
input		I_dac;
	
	
output		O_first_xfer_cycle;
output		O_second_xfer_cycle;
output		O_do_disconnect;
output		O_addr_par;
output		O_addr_par64;
output		O_latched_end_xfer;	
								
reg			O_first_xfer_cycle;
reg			O_second_xfer_cycle;
reg			O_do_disconnect;
reg			O_addr_par;
reg			O_addr_par64;
reg			O_latched_end_xfer;

//internal wires
wire			ctl_frst_cycle;
	
assign	ctl_frst_cycle = ((~I_GNT_N)&&(I_FRAME_N)&&(I_IRDY_N)&&(I_val_addr_lo)); 						

always @(posedge I_CLK or negedge I_RST_N)	//first cycle output assignment
begin		
	if(~I_RST_N)		// Reset the outputs
		O_first_xfer_cycle	<=  1'b0;
	else if(ctl_frst_cycle)	// Check for validity of the first cycle
		O_first_xfer_cycle	<=  1'b1;
	else
		O_first_xfer_cycle	<=  1'b0;
end

always @(posedge I_CLK or negedge I_RST_N)	//second cycle output assignment
begin		
	if(~I_RST_N)		// Reset the outputs
		O_second_xfer_cycle	<=  1'b0;
	else if(O_first_xfer_cycle & I_dac)	// Check for validity of the second cycle
		O_second_xfer_cycle	<=  1'b1;
	else
		O_second_xfer_cycle	<=  1'b0;
end

always @(posedge I_CLK or negedge I_RST_N)	//disconnect output assignment
begin		
	if(~I_RST_N)		// Reset the outputs
		O_do_disconnect	<=  1'b0;
	else if(O_first_xfer_cycle & (|I_latched_addr_lo[1:0]))	// Check for validity of the disconnect
		O_do_disconnect	<=  1'b1;
	else
		O_do_disconnect	<=  1'b0;
end

always @(posedge I_CLK or negedge I_RST_N)		//address parity calculation
begin
	if(~I_RST_N)
		O_addr_par		   <= 1'b0;
	else if(O_first_xfer_cycle)
		O_addr_par         <= (^{I_latched_addr_lo,I_latched_cbe_lo});
	else
		O_addr_par         <= 1'b0;
end	

always @(posedge I_CLK or negedge I_RST_N)		//address parity calculation for 64 bit
begin
	if(~I_RST_N)
		O_addr_par64	   <= 1'b0;
	else if(O_second_xfer_cycle)
		O_addr_par64	   <= (^{I_latched_addr_hi,I_latched_cbe_hi});
	else
		O_addr_par64	   <= 1'b0;
end

always @(posedge I_CLK or negedge I_RST_N)		//latch input signal
begin
	if(~I_RST_N)
		O_latched_end_xfer	<= 1'b0;
	else
		O_latched_end_xfer <= 	I_itr_end_xfer;
end

endmodule

