/*=========================================================================
   CLPCI core                                        Chiplogic confidential
   
   File name            : itr_cmd_decode.v
   Created on           : 8 June 1999
   Function             : Command decoder for the PCI initiator
   Author               : Satish Rathnam
   Modified             : Pramod G : on 22 June 1999
   Revision             : 
   Verified             : 
  	 
   Instantiated in      : itr_top.v

   Instantiates         : 
   Synthesis check      : 
=========================================================================*/
/*

   CBE_n[3:0] 

   0000		Reserved
   0001		Reserved
   0010		I/O read
   0011		I/O write
   0100		Reserved
   0101		Reserved
   0110		Memory Read
   0111		Memory Write
   1000		Reserved
   1001		Reserved
   1010		Reserved
   1011		Reserved
   1100		Reserved
   1101		Dual Address Cycle
   1110		Reserved
   1111		Reserved
*/
`include "itr_header.v"

module itr_cmd_decode (
                          I_CLK ,
                          I_RST_N ,
                          I_app_clk ,
                          I_cbe_lo ,
                          I_cbe_hi ,
                          I_itr_end_xfer,
                          I_itr_a2c_req32 ,                   
                          I_itr_a2c_req64 ,
                          I_val_addr_lo ,
                          I_val_addr_hi ,
                          I_ad_lo ,
                          I_ad_hi ,
                          I_itr_enb ,
                     
                          O_io_rd  ,
                          O_io_wr  ,
                          O_mem_rd ,
                          O_mem_wr ,
                          O_dac ,
                          O_req_32 ,
                          O_req_64 ,
                          O_error,
                          O_latched_cbe_lo ,
                          O_latched_cbe_hi ,
                          O_latched_addr_lo ,
                          O_latched_addr_hi
                      );


input		    I_RST_N ;
input 		    I_CLK ;
input 		    I_app_clk ;
input           I_itr_end_xfer;
input 		    I_itr_a2c_req32 ;
input           I_itr_a2c_req64 ;
input           I_val_addr_lo;    
input           I_val_addr_hi;
input   	    I_itr_enb;
input [`CBE_HALF_WIDTH-1:0]	I_cbe_lo;
input [`CBE_HALF_WIDTH-1:0]	I_cbe_hi;
input [`DATA_HALF_WIDTH-1:0]	I_ad_lo;
input [`DATA_HALF_WIDTH-1:0]	I_ad_hi;

output 		O_io_rd ;
output		O_io_wr ;
output		O_mem_rd ;
output		O_mem_wr ;
output		O_dac ;
output		O_error ;
output		O_req_32 ;
output		O_req_64 ;
output [ `CBE_HALF_WIDTH-1:0]   O_latched_cbe_lo ;
output [ `CBE_HALF_WIDTH-1:0]   O_latched_cbe_hi ;
output [`DATA_HALF_WIDTH-1:0]   O_latched_addr_lo ;
output [`DATA_HALF_WIDTH-1:0]   O_latched_addr_hi ;

reg             req_32;
reg             req_64;
reg             err;

reg [4:0]   	decoded_cmd ; 

reg [3:0]       registered_cmd;    
//reg             registered_req32 ;
//reg             registered_req64 ;

reg [`CBE_HALF_WIDTH-1:0]      latched_cbe_lo ;
reg [`CBE_HALF_WIDTH-1:0]      latched_cbe_hi ;
reg [`DATA_HALF_WIDTH-1:0]     latched_addr_lo;
reg [`DATA_HALF_WIDTH-1:0]     latched_addr_hi;

reg error;

assign O_latched_cbe_lo   = latched_cbe_lo ;
assign O_latched_cbe_hi   = latched_cbe_hi ;
assign O_latched_addr_lo  = latched_addr_lo ;
assign O_latched_addr_hi  = latched_addr_hi ;

assign O_req_32   = req_32;
assign O_req_64   = req_64;


assign O_dac    = decoded_cmd[4] ;
assign O_mem_wr = decoded_cmd[3] ;
assign O_mem_rd = decoded_cmd[2] ;
assign O_io_wr  = decoded_cmd[1] ;
assign O_io_rd  = decoded_cmd[0] ;
assign O_error  = error;

always@(posedge I_app_clk or negedge I_RST_N)
if (I_RST_N == 1'b0)
    error   = 1'b0; 
else
    error   = err; 


always@(posedge I_CLK or negedge I_RST_N)
begin
if (I_RST_N == 1'b0)
   begin 
      
         req_32           <= 1'b0;
         req_64           <= 1'b0;
         registered_cmd   <= 4'b0000 ;
    
   end
else if(I_itr_end_xfer)
   begin
        registered_cmd    <= 4'b0000 ;
        req_64            <= 1'b0; 
        req_32            <= 1'b0;
           
     end       
else
   begin
        registered_cmd    <= I_cbe_lo; 
        req_32            <= I_itr_a2c_req32;  
        if(I_itr_a2c_req64)
           req_64         <= 1'b1;
        else   
           req_64         <= req_64; 
   end 
   
end

/*
//Latching  cbe_lo and cbe_hi

always@(posedge I_CLK or negedge I_RST_N)
begin

if (I_RST_N == 1'b0)
   begin 
        latched_cbe_lo <= 4'b0 ;      
        latched_cbe_hi <= 4'b0 ;
   end
else
   begin            
    
      if(req_32 || req_64) 
        latched_cbe_lo <= I_cbe_lo;
        latched_cbe_hi <= I_cbe_hi;
   
    end 
end
*/

//Latching address,cbe

always@(posedge I_CLK or negedge I_RST_N)
begin
  
  if (I_RST_N == 1'b0 )

     begin
         latched_addr_lo <= 32'b0;
         latched_addr_hi <= 32'b0;
         latched_cbe_lo  <= 4'b0 ;      
         latched_cbe_hi  <= 4'b0 ;
     end 

  else if(I_itr_enb && (I_itr_a2c_req32 || I_itr_a2c_req64 )) // changed req32 & req 64
     begin
        if(I_val_addr_lo)                   // modified
            begin
            latched_addr_lo <= I_ad_lo;
            latched_cbe_lo  <= I_cbe_lo;
            end
        if(I_val_addr_hi)                   // modified  
            begin     
            latched_addr_hi <= I_ad_hi;
            latched_cbe_hi  <= I_cbe_hi;
            end
     end            
  else 
     begin    
         latched_addr_lo <= 32'b0;
         latched_addr_hi <= 32'b0;
         latched_cbe_lo  <= 4'b0 ;      
         latched_cbe_hi  <= 4'b0 ;
                       
     end     
   
end

//Command generation

always@(registered_cmd)
begin
   case (registered_cmd)

      4'b0010 :
        begin 
	       decoded_cmd <= 5'b00001 ;
	       err         <= 1'b0 ;
        end
      4'b0011 :
        begin 
		   decoded_cmd <= 5'b00010 ;
	       err         <= 1'b0 ;
	    end       	
      4'b1110 :
        begin  
       	   decoded_cmd <= 5'b00100 ;
           err         <= 1'b0 ;
	    end
      4'b1111 :
        begin
    	   decoded_cmd <= 5'b01000 ;
	   	   err         <= 1'b0 ;
	    end 	
      4'b1101 :
        begin
		   decoded_cmd <= 5'b10000 ;
		   err         <= 1'b0 ;
        end	
      default : 
        begin
	       decoded_cmd <= 5'b00000 ;
		   err         <= 1'b1 ;
        end
   endcase  
end

endmodule 












