/*****************************************************************************
		Company     : chiplogic 
		File        : itr_top_glue.v
		Created on  : 16th june 1999
		Created by  : Pramod G , Satish
		Description : This includes the frame_sm & itr_sm.
		              This module generates frame,irdy & their output enables
                      This module also generates O_disconnect,O_REQ_N.O_REQ64 
                      signals  & data and address output enables.
******************************************************************************/

`include "itr_header.v"

module itr_engine
                 (
                  I_RST_N,
		  I_CLK,

                  I_FRAME_N, 
                  I_IRDY_N,
                  I_DEVSEL_N,      // from tgt
                  I_TRDY_N,
                  I_STOP_N,
                  I_ACK64_N,
                  I_GNT_N,
	          I_retry_disc,
	          I_itr_end_xfer,
	          I_latched_addr_lo,
	          I_latched_addr_hi,
                  I_val_addr_lo,
	          I_val_addr_hi,
	          I_latched_cbe_hi,
	          I_latched_cbe_lo,
	          I_valid_req32,
	          I_valid_req64,
                  I_io_rd,
                  I_io_wr,
                  I_mem_rd,
                  I_mem_wr,
	          I_dac,
	          I_first_xfer_cycle,
	          I_second_xfer_cycle,
	          I_iwf_full,
	          I_iwf_empty,	// initiator write FIFO empty
                  I_irf_empty,
  	          I_irf_full,	// initiator read FIFO full
                  I_data_present,
                  I_mem_spc_enb,
                  I_io_spc_enb,
	          I_itr_enb,
	          I_do_disconnect,
		          
	          O_disconnect,
	          O_REQ_N,
	          O_REQ64_N,
 	          O_frame_N,     
	          O_oe_frame,
	          O_irdy_N,
	          O_oe_irdy,
	          O_trdy_N,
	          O_devsel_N,
	          O_incr_iwf_wrptr,
	          O_incr_irf_rdptr,
	          O_oe_par,
	          O_oe_par64,
	          O_oe_AD_lo,
	          O_oe_AD_hi,  // later change this ( set in dac only) 
                  O_oe_ADDR_lo,
                  O_oe_ADDR_hi,  
                  O_incapable64,
	          O_pci_cbe_lo,
	          O_pci_cbe_hi,
	          O_pci_addr_lo,
	          O_pci_addr_hi,
	          O_set_tgt_abort_detected,
	          O_set_itr_abort_detected,
	          O_wr_mode                 // new
				 
               ); 
               
parameter  lat_timer_val  = 4'b0111; // check whether 8 or 12 or 16 -- now 8 cycles              

//states of itr_sm
parameter  IDLE1          = 4'b0000 ;  // 0
parameter  WR_INIT_LAT    = 4'b0001 ;  // 1
parameter  WR_XFER        = 4'b0011 ;  // 3
parameter  WR_WAIT        = 4'b0010 ;  // 2
parameter  START_RD_XFER  = 4'b1000 ;  // 8
parameter  CHK_RD_FIFO    = 4'b1001 ;  // 9
parameter  RD_XFER        = 4'b1011 ;  // B
parameter  RD_WAIT        = 4'b1010 ;  // A 
parameter  LAST_CYC       = 4'b1110 ;  // E

//states of frame_sm
parameter  IDLE2	      = 3'b000;    // 0
parameter  ASSERT	      = 3'b001;    // 1
parameter  DAC  	      = 3'b010;    // 2
parameter  DEASSERT	      = 3'b011;    // 3
parameter  DATA_XFER_1	  = 3'b100;    // 4
parameter  DATA_XFER_2	  = 3'b101;    // 5

input	I_CLK;
input	I_RST_N;
input	I_DEVSEL_N;
input	I_TRDY_N;
input	I_IRDY_N;
input	I_FRAME_N;
input	I_STOP_N;
input	I_ACK64_N;
input	I_GNT_N;
input	I_itr_end_xfer;
input	I_val_addr_lo;
input	I_val_addr_hi;
input	I_retry_disc;
input	I_itr_enb;					
input	I_dac;

//input   I_new_itr_cmd ;
//input   I_itr_wr ;
//input   I_itr_rd ;

input [`DATA_HALF_WIDTH-1:0]  I_latched_addr_lo;
input [`DATA_HALF_WIDTH-1:0]  I_latched_addr_hi;
input [`CBE_HALF_WIDTH-1:0]   I_latched_cbe_hi;
input [`CBE_HALF_WIDTH-1:0]   I_latched_cbe_lo;
input   I_valid_req32;
input   I_valid_req64;
input   I_io_rd;
input   I_io_wr;
input   I_mem_rd;
input   I_mem_wr;
input   I_first_xfer_cycle;
input   I_second_xfer_cycle;
input   I_iwf_full;
input   I_iwf_empty;	// initiator write FIFO empty
input   I_irf_empty;
input   I_irf_full;	// initiator read FIFO full
input   I_data_present;
input   I_mem_spc_enb;
input   I_io_spc_enb;
input   I_do_disconnect;

output  O_irdy_N ;    
output  O_oe_irdy ;    
output  O_oe_AD_lo ;
output  O_oe_AD_hi ;
output  O_oe_ADDR_lo;//new
output  O_oe_ADDR_hi;//new
output  O_incapable64;//new
output  O_incr_iwf_wrptr ;
output  O_incr_irf_rdptr ;

output	O_frame_N;
output	O_oe_frame;
output	O_set_tgt_abort_detected;
output	O_set_itr_abort_detected;
output  O_disconnect;
output  O_REQ_N;
output  O_REQ64_N;
output  O_oe_par;
output  O_oe_par64;
output  O_trdy_N;
output  O_devsel_N;


output [`CBE_HALF_WIDTH-1:0]  O_pci_cbe_lo;
output [`CBE_HALF_WIDTH-1:0]  O_pci_cbe_hi;
output [`DATA_HALF_WIDTH-1:0] O_pci_addr_lo;
output [`DATA_HALF_WIDTH-1:0] O_pci_addr_hi;

output 	O_wr_mode;//new	          

reg [`CBE_HALF_WIDTH-1:0] pci_cbe_lo;
reg [`CBE_HALF_WIDTH-1:0] pci_cbe_hi;
reg [`DATA_HALF_WIDTH-1:0] pci_addr_lo;
reg [`DATA_HALF_WIDTH-1:0] pci_addr_hi;

//internal registers wires used in itr_sm 
reg [3:0] present_state ;
reg [3:0] next_state ; 
reg       irdy_N ; // internal sig
reg       O_incr_iwf_wrptr ;
reg       O_incr_irf_rdptr ;

reg       oe_irdy ;
reg       oe_AD_lo ;
reg       oe_AD_hi ;

reg       O_oe_ADDR_lo;//new
reg       O_oe_ADDR_hi;//new

reg       new_itr_cmd;
reg       itr_rd; 
reg       itr_wr; 
reg       O_wr_mode;

// use for latency timers in itr_sm 
reg [3:0] cnt_init_8 ; 
reg [3:0] cnt_wait_8 ;

reg       O_frame_N;
reg       O_oe_frame;
reg       O_set_tgt_abort_detected;
reg       O_set_itr_abort_detected;
reg       O_REQ_N;
reg       O_REQ64_N;

reg       set_tgt_abort_detected;
reg       set_itr_abort_detected;

reg       end_xfer_flag;

//internal registers wires used in frame_sm 
reg [2:0] pstate;
reg [2:0] nstate;
reg [2:0] cnt6 ; 
reg    	  latch_frame;
reg  	  latch_devsel;
reg  	  latch_trdy;
reg  	  latch_irdy;
reg  	  latch_gnt;
reg       latched_end_xfer;
reg       end_xfer_1;
reg       O_incapable64;//new if tgt is incapable to receive 64 bit data at a time
reg       disconnect; // new
reg       latched_GNT_N;

reg [3:0] cnt_xfer_8;//added for latency timer
   
wire	  gnt;
wire      end_xfer;
wire      end_xfer_2;

wire[3:0] cnt_lat;
//added for latency timer
assign cnt_lat = ((cnt_init_8 >= lat_timer_val) | (cnt_wait_8 >= lat_timer_val) | (cnt_xfer_8 >= lat_timer_val));

assign end_xfer_2 = (latched_GNT_N & cnt_lat) ? 1'b1 : 1'b0;
assign end_xfer = end_xfer_1 | I_itr_end_xfer | end_xfer_2; 

assign O_oe_irdy  = oe_irdy;
assign O_irdy_N   = irdy_N ;
assign O_oe_AD_lo = oe_AD_lo ;
assign O_oe_AD_hi = oe_AD_hi ;

assign O_pci_cbe_lo  = pci_cbe_lo; 
assign O_pci_cbe_hi  = pci_cbe_hi;
assign O_pci_addr_lo = pci_addr_lo;
assign O_pci_addr_hi = pci_addr_hi;

assign O_disconnect = (I_do_disconnect | disconnect);

assign gnt = (~I_GNT_N & I_itr_enb);

always@(negedge I_RST_N or posedge I_CLK)
begin
   if(I_RST_N == 1'b0)
      latched_GNT_N <= 1'b1;
   else 
      latched_GNT_N <= I_GNT_N;

end

//REQ generation
always@(negedge I_RST_N or posedge I_CLK)
begin
   if(I_RST_N == 1'b0)
      begin
      O_REQ_N   <= 1'bz;
      end   
   else if((~O_frame_N) | (I_STOP_N == 1'b0 & I_DEVSEL_N == 1'b0 & I_TRDY_N == 1'b1))
      begin
      O_REQ_N   <= 1'b1;
      end   
   else
      begin
      O_REQ_N   <= ~(I_valid_req32 & I_itr_enb);
      end   
end

//REQ64 generation
always@(negedge I_RST_N or posedge I_CLK)
begin
   if(I_RST_N == 1'b0)
      begin
      O_REQ64_N <= 1'bz;
      end   
   else 
      begin
      O_REQ64_N <= O_frame_N & I_valid_req64 & I_itr_enb;
      end
   
end

always@(negedge I_RST_N or posedge I_CLK)
begin
   if (I_RST_N == 1'b0) 
      begin
         pci_cbe_lo <= 4'b0;
         pci_cbe_hi <= 4'b0;
         pci_addr_lo <= 32'b0;
         pci_addr_lo <= 32'b0;
      end
   else
      begin
         if(I_valid_req64) //& I_val_addr_hi) // I_val_addr_hi needed?
            begin
               pci_cbe_lo  <= I_latched_cbe_lo;
               pci_cbe_hi  <= I_latched_cbe_hi;
               pci_addr_lo <= I_latched_addr_lo;
               pci_addr_hi <= I_latched_addr_hi;
            end
         else if(I_valid_req32) // & I_val_addr_lo) // I_val_addr_lo needed?
            begin
               pci_cbe_lo  <= I_latched_cbe_lo;
               pci_cbe_hi  <= 4'b0;
               pci_addr_lo <= I_latched_addr_lo;
               pci_addr_hi <= 32'b0;
            end   
      end
end               

// end_xfer_1
always@(negedge I_RST_N or posedge I_CLK) // check
begin
   if (I_RST_N == 1'b0) 
      begin 
      end_xfer_1 <= 1'b0;
      latched_end_xfer <= 1'b0;
      end
   else 
      begin
      if( I_io_rd | I_mem_rd )       // (itr_rd) changed
         begin
         end_xfer_1 <= I_itr_end_xfer;
         latched_end_xfer <= 1'b0;
         end   
      else if( I_io_wr | I_mem_wr )  // (itr_wr) changed
         begin
         if(O_incr_iwf_wrptr)
            end_xfer_1 <= (latched_end_xfer || I_itr_end_xfer);         
         else if(~latched_end_xfer)   
            begin
            latched_end_xfer <= I_itr_end_xfer;
            end
      end
end    
         end


always@(negedge I_RST_N or posedge I_CLK) // check
// new_itr_cmd
   if (I_RST_N == 1'b0) 
      new_itr_cmd <= 1'b0;
   else if( (I_first_xfer_cycle & ~I_dac) | I_second_xfer_cycle)   
      new_itr_cmd <= 1'b1;
   else   
      new_itr_cmd <= 1'b0;


// Generate itr_rd whenever there is a read command intended for 
// this master (io or memory) Similarly itr_wr.
// It does not matter whether it is a  memory or io operation .

always@(negedge I_RST_N or posedge I_CLK)
begin

if (I_RST_N == 1'b0) 
   begin 
      itr_rd <= 1'b0 ;
      itr_wr <= 1'b0 ;
   end 
else if(end_xfer)
   begin 
      itr_rd <= 1'b0 ;
      itr_wr <= 1'b0 ;
   end 

else 
   begin 

   if ((I_first_xfer_cycle) & (I_dac == 1'b0)) // SAC commands 
      begin 
         if( (I_io_rd) | (I_mem_rd) ) 
            itr_rd <= 1'b1 ;
         else if ( (I_io_wr) | (I_mem_wr) )
            itr_wr <= 1'b1 ;
         else 
            begin 
               itr_rd <= 1'b0 ;
               itr_wr <= 1'b0 ;
            end  
      end 
   else if (I_second_xfer_cycle)  
      begin 
         if (I_mem_rd) 
            itr_rd <= 1'b1 ;
         else if (I_mem_wr)
            itr_wr <= 1'b1 ; 
      end 
   else 
      begin 
         itr_rd <= 1'b0 ;
         itr_wr <= 1'b0 ;
      end 
   end 
    
end 
   

// frame_sm
///////////
always @(pstate or gnt or I_val_addr_lo  or I_FRAME_N or I_IRDY_N or I_TRDY_N
         or end_xfer   or I_dac     or I_DEVSEL_N or I_STOP_N or I_retry_disc
         or I_ACK64_N  or cnt6  or I_valid_req64)
		 
begin

	case (pstate)
	
	IDLE2:
	
				if(gnt & I_val_addr_lo & I_FRAME_N & I_IRDY_N)// idle state
					nstate <= ASSERT;
				else
					nstate <= IDLE2;
					
	ASSERT:
				if(end_xfer)
					nstate <= DEASSERT;
				else if (I_dac)
					nstate <= DAC;
				else 
					nstate <= DATA_XFER_1;
					
	DAC:
				if(end_xfer)
					nstate <= DEASSERT;
				else
					nstate <= DATA_XFER_1;
					
	DEASSERT:
	            if( (I_DEVSEL_N != 1'b0) | end_xfer)
	               nstate  <= IDLE2; 
                else if(~I_IRDY_N & ~I_TRDY_N)   // back to back
				   begin
					if(gnt & I_val_addr_lo)
						nstate <= ASSERT;
					else 
						nstate <= IDLE2;
				   end		
				else    
					nstate <= DEASSERT;
					
    DATA_XFER_1:
                if(end_xfer)
					nstate <= DEASSERT;
				else if(I_DEVSEL_N & (cnt6 == 3'b101) ) //itr_abort
				    begin
					nstate <= DEASSERT;
					$display("came - itr abort1");
					end
				else if((~I_DEVSEL_N & ~I_STOP_N & I_TRDY_N) | (I_retry_disc))//retry
					nstate <= ASSERT;
				else if(~I_DEVSEL_N & ~I_STOP_N & ~I_TRDY_N) //Disconnect A,B
				    begin
					nstate <= DEASSERT;
                    $display("came5");
					end
				else if(I_DEVSEL_N & ~I_STOP_N & I_TRDY_N)// tgt_abort
					nstate <= DEASSERT;
				else if(~I_DEVSEL_N & end_xfer)
					nstate <= DEASSERT;
				else if(~I_DEVSEL_N & I_ACK64_N & I_valid_req64)
					nstate <= DATA_XFER_2;
				else if(~I_DEVSEL_N)                      // check  if(~I_DEVSEL_N) needed?
					nstate <= DATA_XFER_1;
				else if(cnt6 == 3'b101) 	
					nstate <= DEASSERT;
                else
					nstate <= DATA_XFER_1;

    DATA_XFER_2:
                if(end_xfer)
					nstate <= DEASSERT;
				else if((~I_DEVSEL_N & ~I_STOP_N & I_TRDY_N) | (I_retry_disc))
					nstate <= ASSERT;
				else if(~I_DEVSEL_N & ~I_STOP_N & ~I_TRDY_N)
					nstate <= DEASSERT;
				else if(~I_STOP_N & I_DEVSEL_N & I_TRDY_N)
					nstate <= DEASSERT;
				else if(~I_DEVSEL_N & end_xfer)
					nstate <= DEASSERT;
				else if(~I_DEVSEL_N)                       // check if(~I_DEVSEL_N) needed?
					nstate <= DATA_XFER_1;
				else if(cnt6 == 3'b101) 	
					nstate <= DEASSERT;
                else
					nstate <= DATA_XFER_1;

	default:

				nstate <= IDLE2;
					
endcase

end

always @(pstate or I_IRDY_N or I_TRDY_N or I_DEVSEL_N or I_STOP_N or cnt6 
         or end_xfer)

begin
	case(pstate)
	IDLE2:
			begin
				O_frame_N  <= 1'b1;
				O_oe_frame <= 1'b0;
				O_oe_ADDR_lo <= 1'b0;//new
				O_oe_ADDR_hi <= 1'b0;//new
                O_incapable64  <= 1'b0;//new
				set_itr_abort_detected <= 1'b0;
				set_tgt_abort_detected <= 1'b0;
			end
	ASSERT:
			begin
				O_frame_N  <= 1'b0;
				O_oe_frame <= 1'b1;
				O_oe_ADDR_lo <= 1'b1;//new
				O_oe_ADDR_hi <= 1'b0;//new -- modified
                O_incapable64  <= 1'b0;//new
				set_itr_abort_detected <= 1'b0;
				set_tgt_abort_detected <= 1'b0;
			end
	DAC:
			begin
				O_frame_N  <= 1'b0;
				O_oe_frame <= 1'b1;
				O_oe_ADDR_lo <= 1'b1;//new
				O_oe_ADDR_hi <= 1'b1;//new
                O_incapable64  <= 1'b0;//new
				set_itr_abort_detected <= 1'b0;
				set_tgt_abort_detected <= 1'b0;
			end

	DEASSERT:
			begin
				O_oe_frame <= 1'b1;
				O_oe_ADDR_lo <= 1'b0;//new
				O_oe_ADDR_hi <= 1'b0;//new
                O_incapable64  <= 1'b0;//new
				set_itr_abort_detected <= 1'b0;
				set_tgt_abort_detected <= 1'b0;
				if(~I_IRDY_N | end_xfer )          // & ~I_TRDY_N removed
				    begin
					O_frame_N <= 1'b1;
					end
                else if(I_DEVSEL_N == 1'b0)
				    begin
					O_frame_N <= 1'b0;
					end
				else	
					O_frame_N <= 1'b1;
			end

	DATA_XFER_1:
			begin
				O_oe_frame <= 1'b1;
      			O_frame_N  <= 1'b0;
				O_oe_ADDR_lo <= 1'b0;//new
				O_oe_ADDR_hi <= 1'b0;//new
                O_incapable64  <= 1'b0;//new
				if(I_DEVSEL_N & (cnt6 == 3'b101))
				begin
					set_itr_abort_detected <= 1'b1;
					set_tgt_abort_detected <= 1'b0;
					$display("came - itr abort2");
				end
				else if(~I_STOP_N & I_DEVSEL_N & I_TRDY_N)
				begin
					set_itr_abort_detected <= 1'b0;
					set_tgt_abort_detected <= 1'b1;
				end
				else
				begin
					set_itr_abort_detected <= 1'b0;
					set_tgt_abort_detected <= 1'b0;
				end
			end
	DATA_XFER_2:
			begin
				O_oe_frame <= 1'b1;
				O_frame_N  <= 1'b0;
				O_oe_ADDR_lo <= 1'b0;//new
				O_oe_ADDR_hi <= 1'b0;//new
                                O_incapable64  <= 1'b1;//new
				if(I_DEVSEL_N & (cnt6 == 3'b101))
				begin
					set_itr_abort_detected <= 1'b1;
					set_tgt_abort_detected <= 1'b0;
				end
				else if(~I_STOP_N & I_DEVSEL_N & I_TRDY_N)
				begin
					set_itr_abort_detected <= 1'b0;
					set_tgt_abort_detected <= 1'b1;
				end
				else
				begin
					set_itr_abort_detected <= 1'b0;
					set_tgt_abort_detected <= 1'b0;
				end
			end
	default:
			begin
				O_frame_N  <= 1'b1;
				O_oe_frame <= 1'b0;
				O_oe_ADDR_lo <= 1'b0;//new
				O_oe_ADDR_hi <= 1'b0;//new
                O_incapable64  <= 1'b0;//new
				set_itr_abort_detected <= 1'b0;
				set_tgt_abort_detected <= 1'b0;
			end	
endcase

end

// Once I_FRAME_N goes low,& if counter reaches 5
// it stays in 5 till I_FRAME_N goes high.
always @(posedge I_CLK or  negedge I_RST_N)
begin
	if(~I_RST_N)
		cnt6 <= 3'b0;
	else if(~I_FRAME_N && (cnt6 != 3'b101) )
		cnt6 <= cnt6 + 1;
	else if(~I_FRAME_N)
		cnt6 <= cnt6;  // check whether cnt6 <= cnt6 is better
	else
        cnt6 <= 3'b0;	
end


always @(posedge I_CLK or negedge I_RST_N)
begin
	if(~I_RST_N)
        begin
		pstate <= IDLE2;
		present_state <= IDLE1;
        end
	else
        begin
		pstate <= nstate;
		present_state <= next_state;
        end
end 
         


// itr_sm
/////////
always@(present_state or I_DEVSEL_N or new_itr_cmd or itr_wr or itr_rd or 
        I_irf_full or I_iwf_empty or I_TRDY_N or cnt_init_8 or cnt_wait_8 or I_FRAME_N)
begin 


   case (present_state)

   IDLE1 : 

      if ((I_DEVSEL_N == 1'b0) && (new_itr_cmd))

         begin 

         if (itr_wr) 
            next_state <= WR_INIT_LAT ;
         else if (itr_rd)
            next_state <= START_RD_XFER ;
         else 
            next_state <= IDLE1 ;
         end 

      else
         next_state <= IDLE1 ; 

   WR_INIT_LAT :
          
      if (cnt_init_8 == lat_timer_val ) // check value later  - 12 /16 ??
         next_state <= LAST_CYC ;
      else 
        
         if (I_iwf_empty) 
            next_state <= WR_INIT_LAT ; 
         else
            next_state <= WR_XFER ;
          
   WR_XFER: 

      begin

         if (I_FRAME_N)
               next_state <= LAST_CYC ;

         else if ((I_TRDY_N == 1'b0) && (I_iwf_empty == 0) )
	          
            next_state <= WR_XFER ;
         else  
                
	    next_state <= WR_WAIT ;

      end 
   
   WR_WAIT :

      if (cnt_wait_8 == lat_timer_val)  // check value  - now 7 clocks
         next_state <= LAST_CYC ;

      else if ((I_TRDY_N ) || (I_iwf_empty))
            
         next_state <= WR_WAIT ;
      else
         next_state <= WR_XFER ;

   START_RD_XFER :
      next_state <= CHK_RD_FIFO ;

   CHK_RD_FIFO :
      
      if (cnt_init_8 == lat_timer_val ) // check value later  - 12 /16 ??
         next_state <= LAST_CYC ;
      else 
        
         if (I_irf_full ) 
            next_state <= CHK_RD_FIFO ;
         else
            next_state <= RD_XFER ;
 
   RD_XFER :

         if (I_FRAME_N ==1'b1)
               next_state <= LAST_CYC ;

         else if ((I_TRDY_N == 1'b0) && (I_irf_full == 1'b0) )

                  next_state <= RD_XFER ;

         else
                  next_state <= RD_WAIT ;
   
   RD_WAIT : 

      if (cnt_wait_8 == lat_timer_val)  // check value  - now 7 clocks
         next_state <= LAST_CYC ;

      else if (I_TRDY_N  | I_irf_full)
            
         next_state <= RD_WAIT ;
      else
         next_state <= RD_XFER ;

   LAST_CYC : // all outputs should be driven with their de-asserted value
      
      next_state <= IDLE1 ;
	  

   default : // 
 
      next_state <= IDLE1; // full case - parallel case etc required ??

   endcase 
end


// latency counters for initiator initial latency and in-between-data latency
always@(posedge I_CLK or negedge I_RST_N)
begin 
if (!I_RST_N)
   begin
      cnt_wait_8 <= 0 ;
      cnt_init_8 <= 0 ;
   end 
else
   begin
      case (present_state)
  
      WR_INIT_LAT,
      CHK_RD_FIFO : 
         begin 
         cnt_wait_8 <= 0 ;
         cnt_init_8 <= cnt_init_8 + 1 ;
         cnt_xfer_8 <= 1'b0;
         end 
 
      WR_WAIT,
      RD_WAIT : 
         begin 
         cnt_wait_8 <= cnt_wait_8 + 1 ;
         cnt_init_8 <= 0 ;
         cnt_xfer_8 <= 1'b0;         
         end 
      RD_XFER,
      WR_XFER :
         begin    
         cnt_wait_8 <= 0 ;
         cnt_init_8 <= 0 ;
         cnt_xfer_8 <= cnt_xfer_8 + 1;         
         end

      LAST_CYC:
         begin 
         cnt_wait_8 <= cnt_wait_8;
         cnt_init_8 <= cnt_init_8;
         cnt_xfer_8 <= cnt_xfer_8;
         end 
      
      default : 
         begin 
         cnt_wait_8 <= 0 ;
         cnt_init_8 <= 0 ;
         cnt_xfer_8 <= 0 ;
         end 
      endcase 

/*      case (pstate)
  
      ASSERT,
      DATA_XFER_1,
      DATA_XFER_2 : 
         begin 
//         cnt_wait_8  <= 0 ;
         cnt_init_8 <= cnt_init_8 + 1 ;
         end 
//      DEASSERT:
//         begin 
//         $display("came -deassert , t=%0d",$time);
//         cnt_init_8 <= cnt_init_8;
//         end 
         
//      WR_WAIT,
//      RD_WAIT : 
//         begin 
//         cnt_wait_8  <= cnt_wait_8 + 1 ;
//         cnt_init_8 <= 0 ;         
//         end 

      default : 
         begin 
//         cnt_wait_8  <= 0 ;
         cnt_init_8 <= 0 ;
         end 
      endcase 
*/      
   end   
end  


always@(present_state or I_irf_full or I_iwf_empty)
begin 
   case (present_state)

   IDLE1,
   WR_INIT_LAT:
      begin 
         irdy_N  <= 1'b1 ;
         oe_irdy <= 1'b0 ;

         oe_AD_lo  <= 1'b0 ; 
         oe_AD_hi  <= 1'b0 ;

         O_incr_iwf_wrptr <= 1'b0 ;
         O_incr_irf_rdptr <= 1'b0 ;
      end 

  
   START_RD_XFER,
   CHK_RD_FIFO:
      
      begin 
         irdy_N <= 1'b1 ;
         oe_irdy <= 1'b1 ;

         oe_AD_lo  <= 1'b0 ;
         oe_AD_hi  <= 1'b0 ;

         O_incr_iwf_wrptr <= 1'b0 ;
         O_incr_irf_rdptr <= 1'b0 ;
      end 
     
         

                
   WR_XFER :
      begin
         
         if(I_iwf_empty)
          begin
            irdy_N <= 1'b1 ;
            oe_irdy <= 1'b1 ;
 
            oe_AD_lo  <= 1'b0 ; 
            oe_AD_hi  <= 1'b0 ;
            O_incr_iwf_wrptr <= 1'b0 ;
            O_incr_irf_rdptr <= 1'b0 ;
          end
            
         else
          begin 
            irdy_N <= 1'b0 ;
            oe_irdy <= 1'b1 ;

            oe_AD_lo  <= 1'b1 ; 
            oe_AD_hi  <= 1'b1 ;
            O_incr_iwf_wrptr <= 1'b1 ;
            O_incr_irf_rdptr <= 1'b0 ;
           end 
       end  
    
   RD_XFER:
      begin
         if (I_irf_full)   
             begin 
                irdy_N <= 1'b1 ;
                oe_irdy <= 1'b1 ;

                oe_AD_lo  <= 1'b0 ; 
                oe_AD_hi  <= 1'b0 ;

                O_incr_iwf_wrptr <= 1'b0 ;
                O_incr_irf_rdptr <= 1'b0 ;
             end 

         else 
            begin 
               irdy_N <= 1'b0 ;
               oe_irdy <= 1'b1 ;

               oe_AD_lo  <= 1'b0 ; 
               oe_AD_hi  <= 1'b0 ;
 
               O_incr_iwf_wrptr <= 1'b0 ;
               O_incr_irf_rdptr <= 1'b1 ;
               
            end 

      end 

   LAST_CYC:
      begin 
         irdy_N <= 1'b1 ;
         oe_irdy <= 1'b0 ;

         oe_AD_lo  <= 1'b0 ; 
         oe_AD_hi  <= 1'b0 ;

         O_incr_iwf_wrptr <= 1'b0 ;
         O_incr_irf_rdptr <= 1'b0 ;
      end 


   WR_WAIT: //  should verify 
   // when it is a wait for  data to be available, even though irdy is deasserted,
   // they have to be driven, so their enables should be high
      begin 
         irdy_N <= 1'b1 ;
         oe_irdy <= 1'b1 ;

         oe_AD_lo  <= 1'b1 ; 
         oe_AD_hi  <= 1'b1 ;

         O_incr_iwf_wrptr <= 1'b0 ;
         O_incr_irf_rdptr <= 1'b0 ;
      end 

   RD_WAIT: //  should verify 
   // when it is a wait for  data to be available, even though irdy is deasserted,
   // they have to be driven, so their enables should be high
      begin 
         irdy_N <= 1'b1 ;
         oe_irdy <= 1'b1 ;

         oe_AD_lo  <= 1'b0 ; 
         oe_AD_hi  <= 1'b0 ;

         O_incr_iwf_wrptr <= 1'b0 ;
         O_incr_irf_rdptr <= 1'b0 ;
      end 


   default : 
      begin 
         irdy_N <= 1'b1 ;
         oe_irdy <= 1'b0 ;

         oe_AD_lo  <= 1'b0 ; 
         oe_AD_hi  <= 1'b0 ;

         O_incr_iwf_wrptr <= 1'b0 ;
         O_incr_irf_rdptr <= 1'b0 ;
      end 

   endcase
end

// generate write mode 
always@(present_state)
begin
   case (present_state) 
   
   WR_WAIT, 
   WR_XFER,
   WR_INIT_LAT : O_wr_mode <= 1'b1 ;

   default :     O_wr_mode <= 1'b0 ;

   endcase
end


//generate disconnect
always@(pstate or I_DEVSEL_N or I_STOP_N or I_TRDY_N)
begin
   case (pstate)
 
   DATA_XFER_1,	 
   DATA_XFER_2:
                if(~I_DEVSEL_N & ~I_STOP_N & ~I_TRDY_N) //Disconnect A,B           
                   begin
                   disconnect <= 1'b1;
                   end
                else
                   begin
                   disconnect <= 1'b0;   
                   end
                   
   default    :                    
                   disconnect <= 1'b0;
   endcase
end   


always @(posedge I_CLK or negedge I_RST_N)
if(~I_RST_N)
   begin
   O_set_itr_abort_detected <= 1'b0;
   end
else if(set_itr_abort_detected)   
   begin
   O_set_itr_abort_detected <= 1'b1;
   end
else   
   begin
   O_set_itr_abort_detected <= O_set_itr_abort_detected;
   end


always @(posedge I_CLK or negedge I_RST_N)
if(~I_RST_N)
   begin
   O_set_tgt_abort_detected <= 1'b0;
   end
else if(set_tgt_abort_detected)   
   begin
   O_set_tgt_abort_detected <= 1'b1;
   end
else   
   begin
   O_set_tgt_abort_detected <= O_set_tgt_abort_detected;
   end


endmodule  






