// File Name   : pci_ram16x32_2.v
// Edit History:
// Author           Date           Comments
// Arvind Agrawal   02/17/99       Started
// Description:
// This is a 16 x 32 ram which will be used to make the 16 x 32 FIFO with
// the help of a wrapper on this file
// Module instantiated in this file : None
// Module where this module is instantiated : pci_fifo

//`timescale 1ns / 10ps


module itr_ram16x36 
   ( I_rd_addr,
     I_wr_addr,
     I_wr_data,
     O_rd_data,
     I_we_N,
     I_cs_N,
     I_pwrclr_N,
     I_clk
     );
   
   
   //
   //-------- input declarations
   //
   
   input	I_clk;
   input	I_pwrclr_N;
   input [3:0]	I_rd_addr;      // ram read address
   input [3:0]	I_wr_addr;     // ram write address
   input [35:0]	I_wr_data;         // ram input data
   input	I_cs_N;
   
   input	I_we_N;       // ram write clock
   
   //
   //-------- output declarations
   //
   
   output [35:0]	O_rd_data;          // ram output data
   
   
   //
   //-------- register declarations
   //
   
   reg [35:0]	row_0;
   reg [35:0]	row_1;
   reg [35:0]	row_2;
   reg [35:0]	row_3;
   reg [35:0]	row_4;
   reg [35:0]	row_5;
   reg [35:0]	row_6;
   reg [35:0]	row_7;
   reg [35:0]	row_8;
   reg [35:0]	row_9;
   reg [35:0]	row_10;
   reg [35:0]	row_11;
   reg [35:0]	row_12;
   reg [35:0]	row_13;
   reg [35:0]	row_14;
   reg [35:0]	row_15;
   reg [35:0]	read_mux_a;
   
   wire		row_0_match;
   wire		row_1_match;
   wire		row_2_match;
   wire		row_3_match;
   wire		row_4_match;
   wire		row_5_match;
   wire		row_6_match;
   wire		row_7_match;
   wire		row_8_match;
   wire		row_9_match;
   wire		row_10_match;
   wire		row_11_match;
   wire		row_12_match;
   wire		row_13_match;
   wire		row_14_match;
   wire		row_15_match;
   
   
   wire		row_0_wrstb;
   wire		row_1_wrstb;
   wire		row_2_wrstb;
   wire		row_3_wrstb;
   wire		row_4_wrstb;
   wire		row_5_wrstb;
   wire		row_6_wrstb;
   wire		row_7_wrstb;
   wire		row_8_wrstb;
   wire		row_9_wrstb;
   wire		row_10_wrstb;
   wire		row_11_wrstb;
   wire		row_12_wrstb;
   wire		row_13_wrstb;
   wire		row_14_wrstb;
   wire		row_15_wrstb;
   
   
   
   
   assign O_rd_data = read_mux_a;
   
   
   //
   //-------- read muxes
   //
   
   always @(row_0 or row_1 or row_2 or row_3 or
	    row_4 or row_5 or row_6 or row_7 or
	    row_8 or row_9 or row_10 or row_11 or
	    row_12 or row_13 or row_14 or row_15 or I_rd_addr)
      begin
	 case (I_rd_addr)
	   0  :  read_mux_a  =  row_0;
	   1  :  read_mux_a  =  row_1;
	   2  :  read_mux_a  =  row_2;
	   3  :  read_mux_a  =  row_3;
	   4  :  read_mux_a  =  row_4;
	   5  :  read_mux_a  =  row_5;
	   6  :  read_mux_a  =  row_6;
	   7  :  read_mux_a  =  row_7;
	   8  :  read_mux_a  =  row_8;
	   9  :  read_mux_a  =  row_9;
	   10  :  read_mux_a  =  row_10;
	   11  :  read_mux_a  =  row_11;
	   12  :  read_mux_a  =  row_12;
	   13  :  read_mux_a  =  row_13;
	   14  :  read_mux_a  =  row_14;
	   15  :  read_mux_a  =  row_15;
	 endcase
      end
   
   //
   //-------- write the selected row
   //
   
   assign row_0_match  = I_wr_addr==0 ;
   assign row_1_match  = I_wr_addr==1 ;
   assign row_2_match  = I_wr_addr==2 ;
   assign row_3_match  = I_wr_addr==3 ;
   assign row_4_match  = I_wr_addr==4 ;
   assign row_5_match  = I_wr_addr==5 ;
   assign row_6_match  = I_wr_addr==6 ;
   assign row_7_match  = I_wr_addr==7 ;
   assign row_8_match  = I_wr_addr==8 ;
   assign row_9_match  = I_wr_addr==9 ;
   assign row_10_match = I_wr_addr==10 ;
   assign row_11_match = I_wr_addr==11 ;
   assign row_12_match = I_wr_addr==12 ;
   assign row_13_match = I_wr_addr==13 ;
   assign row_14_match = I_wr_addr==14 ;
   assign row_15_match = I_wr_addr==15 ;
   
   assign	row_0_wrstb  = ~I_we_N & row_0_match; 
   assign	row_1_wrstb  = ~I_we_N & row_1_match; 
   assign	row_2_wrstb  = ~I_we_N & row_2_match; 
   assign	row_3_wrstb  = ~I_we_N & row_3_match; 
   assign	row_4_wrstb  = ~I_we_N & row_4_match; 
   assign	row_5_wrstb  = ~I_we_N & row_5_match; 
   assign	row_6_wrstb  = ~I_we_N & row_6_match; 
   assign	row_7_wrstb  = ~I_we_N & row_7_match; 
   assign	row_8_wrstb  = ~I_we_N & row_8_match; 
   assign	row_9_wrstb  = ~I_we_N & row_9_match; 
   assign	row_10_wrstb = ~I_we_N & row_10_match; 
   assign	row_11_wrstb = ~I_we_N & row_11_match; 
   assign	row_12_wrstb = ~I_we_N & row_12_match; 
   assign	row_13_wrstb = ~I_we_N & row_13_match; 
   assign	row_14_wrstb = ~I_we_N & row_14_match; 
   assign	row_15_wrstb = ~I_we_N & row_15_match; 
   
   
   always @ (posedge I_clk or negedge I_pwrclr_N)
      if (~I_pwrclr_N)
	 row_0 = 36'h0000_0000_0;
      else if (row_0_wrstb==1)
	 row_0 = I_wr_data;
   
   always @ (posedge I_clk or negedge I_pwrclr_N)
      if (~I_pwrclr_N)
	 row_1 = 36'h0000_0000_0;
      else if (row_1_wrstb==1)
	 row_1 = I_wr_data;
   
   always @ (posedge I_clk or negedge I_pwrclr_N)
      if (~I_pwrclr_N)
	 row_2 = 36'h0000_0000_0;
      else if (row_2_wrstb==1)
	 row_2 = I_wr_data;
   
   always @ (posedge I_clk or negedge I_pwrclr_N)
      if (~I_pwrclr_N)
	 row_3 = 36'h0000_0000_0;
      else if (row_3_wrstb==1)
	 row_3 = I_wr_data;
   
   always @ (posedge I_clk or negedge I_pwrclr_N)
      if (~I_pwrclr_N)
	 row_4 = 36'h0000_0000_0;
      else if (row_4_wrstb==1)
	 row_4 = I_wr_data;
   
   always @ (posedge I_clk or negedge I_pwrclr_N)
      if (~I_pwrclr_N)
	 row_5 = 36'h0000_0000_0;
      else if (row_5_wrstb==1)
	 row_5 = I_wr_data;
   
   always @ (posedge I_clk or negedge I_pwrclr_N)
      if (~I_pwrclr_N)
	 row_6 = 36'h0000_0000_0;
      else if (row_6_wrstb==1)
	 row_6 = I_wr_data;
   
   always @ (posedge I_clk or negedge I_pwrclr_N)
      if (~I_pwrclr_N)
	 row_7 = 36'h0000_0000_0;
      else if (row_7_wrstb==1)
	 row_7 = I_wr_data;
   
   always @ (posedge I_clk or negedge I_pwrclr_N)
      if (~I_pwrclr_N)
	 row_8 = 36'h0000_0000_0;
      else if (row_8_wrstb==1)
	 row_8 = I_wr_data;
   
   always @ (posedge I_clk or negedge I_pwrclr_N)
      if (~I_pwrclr_N)
	 row_9 = 36'h0000_0000_0;
      else if (row_9_wrstb==1)
	 row_9 = I_wr_data;
   
   always @ (posedge I_clk or negedge I_pwrclr_N)
      if (~I_pwrclr_N)
	 row_10 = 36'h0000_0000_0;
      else if (row_10_wrstb==1)
	 row_10 = I_wr_data;
   
   always @ (posedge I_clk or negedge I_pwrclr_N)
    if (~I_pwrclr_N)
       row_11 = 36'h0000_0000_0;
    else if (row_11_wrstb==1)
       row_11 = I_wr_data;
   
   always @ (posedge I_clk or negedge I_pwrclr_N)
      if (~I_pwrclr_N)
	 row_12 = 36'h0000_0000_0;
      else if (row_12_wrstb==1)
	 row_12 = I_wr_data;
   
   always @ (posedge I_clk or negedge I_pwrclr_N)
      if (~I_pwrclr_N)
	 row_13 = 36'h0000_0000_0;
      else if (row_13_wrstb==1)
	 row_13 = I_wr_data;
   
   always @ (posedge I_clk or negedge I_pwrclr_N)
      if (~I_pwrclr_N)
	 row_14 = 36'h0000_0000_0;
      else if (row_14_wrstb==1)
	 row_14 = I_wr_data;
   
   always @ (posedge I_clk or negedge I_pwrclr_N)
      if (~I_pwrclr_N)
	 row_15 = 36'h0000_0000;
      else if (row_15_wrstb==1)
	 row_15 = I_wr_data;
   
endmodule // itr_ram16x36

