/**************************************************************************
		Company     : chiplogic 
		File        : Initiator read data path
		Created on  : 10th june 1999
		modified on : 22nd June 1999 : Pramod
		Created by  : D.A.Naga raju
		Description : This a Initiator read data path for PCI controller
					  with asynchronous reset and synchronous data transfer 
                      This also checks the parity of the input data from 
                      PCI side.
***************************************************************************/
`include "itr_header.v"

module itr_rd_dp_pchk
                   (
                    I_CLK,
					I_RST_N,
					I_PAR,//new
					I_PAR64,//new
					I_incr_rdptr,//new
					I_prty_err_resp,//new
					I_in_data_lo,
					I_in_data_hi,
					I_irdy_N,
					I_trdy_N,
					I_devsel_N,
					I_in_cbe_lo,
					I_in_cbe_hi,
					O_itr_rf_data_lo,
					O_itr_rf_data_hi,
					O_itr_rf_cbe_lo,
					O_itr_rf_cbe_hi,
					O_set_detected_prty_err,//new
					O_set_prty_err_reported,//new
					IO_PERR_N//new
					
				);
					
inout		                   IO_PERR_N;
					
input                          I_CLK;
input                          I_RST_N;
input		                   I_PAR;
input		                   I_PAR64;
input                          I_incr_rdptr;
input 	 	                   I_prty_err_resp;

input  [`DATA_HALF_WIDTH-1:0]  I_in_data_lo;
input  [`DATA_HALF_WIDTH-1:0]  I_in_data_hi;
input  [`CBE_HALF_WIDTH-1:0]    I_in_cbe_lo;
input  [`CBE_HALF_WIDTH-1:0]    I_in_cbe_hi;
input                          I_devsel_N;
input                          I_irdy_N;
input                          I_trdy_N;
	
output [`DATA_HALF_WIDTH-1:0]  O_itr_rf_data_lo;
output [`DATA_HALF_WIDTH-1:0]  O_itr_rf_data_hi;
output [`CBE_HALF_WIDTH-1:0]    O_itr_rf_cbe_lo;
output [`CBE_HALF_WIDTH-1:0]    O_itr_rf_cbe_hi;
output		                   O_set_detected_prty_err;
output		                   O_set_prty_err_reported;
	
	//internal registers
reg    [`DATA_HALF_WIDTH-1:0]  O_itr_rf_data_lo;
reg	   [`DATA_HALF_WIDTH-1:0]  O_itr_rf_data_hi;
reg    [`CBE_HALF_WIDTH-1:0]    O_itr_rf_cbe_lo;
reg    [`CBE_HALF_WIDTH-1:0]    O_itr_rf_cbe_hi;
	
reg			calc_parity;//new
reg			calc_parity64;//new
reg			latch_parity;//new
reg			latch_parity64;//new
reg			p_err;//new
reg         O_set_detected_prty_err;	
reg  		O_set_prty_err_reported;

// read data path
always @(posedge I_CLK or negedge I_RST_N)        

	begin
		if(~I_RST_N)
			begin      //reset the output
				O_itr_rf_data_lo <= 32'h00000000;
				O_itr_rf_data_hi <= 32'h00000000;
				O_itr_rf_cbe_lo  <= 4'h0;
				O_itr_rf_cbe_hi  <= 4'h0;
			end

		//check for device selection and ready signals
		else if(~I_devsel_N && ~I_irdy_N && ~I_trdy_N)
			
			begin      //data assignment from input to output
				O_itr_rf_data_lo <= I_in_data_lo;
				O_itr_rf_data_hi <= I_in_data_hi;
				O_itr_rf_cbe_lo  <= I_in_cbe_lo;
				O_itr_rf_cbe_hi  <= I_in_cbe_hi;
			end
	end


// parity check 
assign  IO_PERR_N = ~p_err;	

//calculate parity for the received data when I_incr_rdptr is available
always @(posedge I_CLK or negedge I_RST_N)
begin
    if(~I_RST_N)
        begin
        	calc_parity   <= 1'b0;
		    calc_parity64 <= 1'b0;
		end
	else if(I_incr_rdptr)
		begin
			calc_parity   <= (^{O_itr_rf_data_lo,O_itr_rf_cbe_lo});
			calc_parity64 <= (^{O_itr_rf_data_hi,O_itr_rf_cbe_hi});
//			calc_parity   <= (^{I_in_data_lo,I_in_cbe_lo});
//			calc_parity64 <= (^{I_in_data_hi,I_in_cbe_hi});
			$display("came1");  // ** Remove 
		end	
	else	
        begin
        	calc_parity   <= 1'b0;
		    calc_parity64 <= 1'b0;
			$display("came2");   // ** Remove
		end

end

	//parity check 				
always @(posedge I_CLK or negedge I_RST_N)
begin 
    if(~I_RST_N)
		O_set_detected_prty_err <= 1'b0;
    else if((calc_parity	!= latch_parity) || (calc_parity64 != latch_parity64))
		O_set_detected_prty_err <= 1'b1;
	else		
		O_set_detected_prty_err <= O_set_detected_prty_err;
end
	
	
	
always @(posedge I_CLK or negedge I_RST_N)	//set reported parity and p_err
begin
	 if(~I_RST_N)		//Reset
	 	begin
   		 	p_err					<= 1'b0;
			O_set_prty_err_reported <= 1'b0;
		end
						//check for prty_resp and prty_err
	else if(I_prty_err_resp && ((calc_parity != latch_parity) || (calc_parity64 != latch_parity64)) )
		begin
			p_err					<= 1'b1;
			O_set_prty_err_reported <= 1'b1;
		end
	else				
		begin
			p_err					<= 1'b0;
			O_set_prty_err_reported <= O_set_prty_err_reported;
		end	
end

always @(posedge I_CLK or negedge I_RST_N)
begin
	if(~I_RST_N)		//reset the output 
		begin
			latch_parity    <= 1'b0;
			latch_parity64  <= 1'b0;
		end
	else
		begin			//save the received parity bits
			latch_parity    <= I_PAR;
			latch_parity64  <= I_PAR64;
		end
end
	
endmodule

	