/*=========================================================================
   CLPCI core                                        Chiplogic confidential
   
   File name            : itr_rw_fifo
   Created on           : 11 Jun 99
   Function             : initiator Fifo
   Author               : Rsk
   Revision             : 1.0
   Verified             : no
   Modification History : 

   Instantiated in      : 

   Instantiates         :
   Synthesis check      : done/not done 
=========================================================================*/

// This module instantiates the pci_fifo module and creates a wrapper 
// around it for use in both target read and write modes 

`include "itr_header.v"

module itr_rw_fifo
(

      
      // inputs from pci side
                     I_CLK,                          
                     I_RST_N ,
                     I_app_clk   ,
                     I_fifo_reset , 
                     I_data_val_lo ,
                     I_data_val_hi ,
                     I_itr_rd_nxt ,
                     I_incr_itr_wrptr  ,
                     I_incr_itr_rdptr    ,
                     I_itr_rf_in_data_lo ,
                     I_itr_rf_in_data_hi ,
                     I_itr_rf_in_cbe ,
                     I_ad_lo ,
                     I_ad_hi ,
                     I_cbe ,
                     
                     
                     O_data_present ,//initiator eng.
                     O_irf_empty ,
                     O_irf_full  ,
                     O_iwf_empty ,
                     O_iwf_full ,
                     O_itr_rd_data_avail ,
                     O_irf_cbe ,
                     O_iwf_cbe ,
                     O_iwf_data_lo,
                     O_iwf_data_hi,
                     O_irf_data_lo,
                     O_irf_data_hi


   );


input        I_CLK ;
input        I_RST_N ;
input        I_fifo_reset ;
input [`DATA_HALF_WIDTH-1:0] I_ad_lo ;
input [`DATA_HALF_WIDTH-1:0] I_ad_hi ;
input        I_incr_itr_wrptr ;
input        I_incr_itr_rdptr ;
input [`CBE_FULL_WIDTH-1:0]  I_cbe;

input        I_app_clk ;
input        I_itr_rd_nxt ;
input [`DATA_HALF_WIDTH-1:0] I_itr_rf_in_data_lo;
input [`DATA_HALF_WIDTH-1:0] I_itr_rf_in_data_hi;
input        I_data_val_lo ;
input        I_data_val_hi ;
input [`CBE_FULL_WIDTH-1:0]  I_itr_rf_in_cbe;   

output [`DATA_HALF_WIDTH-1:0] O_iwf_data_lo ;
output [`DATA_HALF_WIDTH-1:0] O_iwf_data_hi ;
output        O_iwf_full ;
output        O_iwf_empty ;

output[`CBE_FULL_WIDTH-1:0]   O_irf_cbe ;
output[`CBE_FULL_WIDTH-1:0]   O_iwf_cbe ;
output        O_itr_rd_data_avail;
output        O_data_present;
output        O_irf_full ;
output        O_irf_empty ;
output [`DATA_HALF_WIDTH-1:0] O_irf_data_lo ;
output [`DATA_HALF_WIDTH-1:0] O_irf_data_hi ;

wire[35:0] ad_cbe_lo;
wire[35:0] ad_cbe_hi;
wire[35:0] itr_rf_in_data_cbe_lo;
wire[35:0] itr_rf_in_data_cbe_hi;

wire[35:0] iwf_data_lo;
wire[35:0] iwf_data_hi;
wire[35:0] irf_data_lo;
wire[35:0] irf_data_hi;

assign O_irf_data_lo  = irf_data_lo[35:4];
assign O_irf_cbe[3:0] = irf_data_lo[3:0];
assign O_irf_data_hi  = irf_data_hi[35:4];
assign O_irf_cbe[7:4] = irf_data_hi[3:0];
assign O_iwf_data_lo  = iwf_data_lo[35:4];
assign O_iwf_cbe[3:0] = iwf_data_lo[3:0];
assign O_iwf_data_hi  = iwf_data_hi[35:4];
assign O_iwf_cbe[7:4] = irf_data_hi[3:0];

assign O_itr_rd_data_avail = (~O_irf_empty) ;

assign ad_cbe_lo = {I_ad_lo,I_cbe[3:0]};
assign ad_cbe_hi = {I_ad_hi,I_cbe[7:4]};
assign itr_rf_in_data_cbe_lo = {I_itr_rf_in_data_lo,I_itr_rf_in_cbe[3:0]};
assign itr_rf_in_data_cbe_hi = {I_itr_rf_in_data_hi,I_itr_rf_in_cbe[7:4]};


itr_fifo iwf_low_bank
   (
    .I_data_in       (ad_cbe_lo),
    .I_we            (I_data_val_lo),  
    .I_re            (I_incr_itr_wrptr),
    .I_pwrclr_N      (I_RST_N),
    .I_fifo_reset    (I_fifo_reset), 
    .O_full          (O_iwf_full) ,
    .O_empty         (O_iwf_empty),
    .O_data_out      (iwf_data_lo),
    .I_clk_wr        (I_app_clk),
    .I_clk_rd        (I_CLK)
    );

itr_fifo iwf_hi_bank
   (
    .I_data_in       (ad_cbe_hi),  
    .I_we            (I_data_val_hi),
    .I_re            (I_incr_itr_wrptr),
    .I_pwrclr_N      (I_RST_N),
    .I_fifo_reset    (I_fifo_reset), 
    .O_full          (O_iwf_full), // this need not be used
    .O_empty         (O_iwf_empty), // for any decision logic
    .O_data_out      (iwf_data_hi),
    .I_clk_wr        (I_app_clk),
    .I_clk_rd        (I_CLK)
    );


itr_fifo irf_low_bank
   (
    .I_data_in       (itr_rf_in_data_cbe_lo),
    .I_we            (I_incr_itr_rdptr),
    .I_re            (I_itr_rd_nxt),
    .I_pwrclr_N      (I_RST_N),
    .I_fifo_reset    (1'b0), //check
    .O_full          (O_irf_full),
    .O_empty         (O_irf_empty),
    .O_data_out      (irf_data_lo),
    .I_clk_wr        (I_CLK),
    .I_clk_rd        (I_app_clk)
    );

itr_fifo irf_hi_bank
   (
    .I_data_in       (itr_rf_in_data_cbe_hi),
    .I_we            (I_incr_itr_rdptr),
    .I_re            (I_itr_rd_nxt),
    .I_pwrclr_N      (I_RST_N),
    .I_fifo_reset    (1'b0), //check
    .O_full          (O_irf_full),
    .O_empty         (O_irf_empty),
    .O_data_out      (irf_data_hi),
    .I_clk_wr        (I_CLK),
    .I_clk_rd        (I_app_clk)
    );


endmodule 













