/*=========================================================================
   CLPCI core                                        Chiplogic confidential
   
   File name            : itr_top.v
   Created on           : 11 June 1999
   Function             : Top level for Initiator 
   Author               : Pramod G & Satish kumar
   Revision             : 1.0
   Verified             : no
   Modification History : Pramod changed some module names on 22 June

   Instantiated in      : CLPCI core

   Instantiates         : 
   Synthesis check      : not done 
=========================================================================*/

// This module envolopes all initiator functions. 

`include "itr_header.v"

module  itr_top
               (
                          // inputs from PCI side
                I_CLK,
                I_RST_N,
                I_GNT_N,
                I_M66EN,
                I_TRDY_N,
                I_STOP_N,
                I_DEVSEL_N,
                I_ACK64_N,
 
                          // inputs from appl. side                
                I_appl_clk,          
                I_ad,
                I_data_val_lo,
                I_data_val_hi,
                I_cbe,
                I_itr_a2c_req32,
                I_itr_a2c_req64,
                I_addr_val_lo,
                I_addr_val_hi,
                I_itr_end_xfer,
                I_itr_rd_nxt,
                I_retry_disc,
                
                          // outputs to PCI side
                O_REQ_N,
                O_REQ64_N,

                          // outputs to appl. side                
                O_data,
                O_data_val_lo,
                O_data_val_hi,
                O_cbe,
                O_64bit_mode,
                O_itr_rd_data_avail,
                O_itr_do_wr_xfer,
                O_disconnect,
                O_error,
                
                          // inouts for PCI side
                IO_AD,
                IO_CBE_N,
                IO_PAR,
                IO_PAR64,
                IO_PERR_N,
                IO_FRAME_N,
                IO_IRDY_N,
               
               );
               
                          // inputs from PCI side
               
input                I_CLK;
input                I_RST_N;
input                I_GNT_N;
input                I_M66EN;
input                I_TRDY_N;
input                I_STOP_N;
input                I_DEVSEL_N;
input                I_ACK64_N;
 
                          // inputs from appl. side                
input                I_appl_clk;
input [63:0]         I_ad;
input                I_data_val_lo;
input                I_data_val_hi;
input [7:0]          I_cbe;
input                I_itr_a2c_req32;
input                I_itr_a2c_req64;
input                I_addr_val_lo;
input                I_addr_val_hi;
input                I_itr_end_xfer;
input                I_itr_rd_nxt;
input                I_retry_disc;
               
                          // outputs to PCI side
output               O_REQ_N;
output               O_REQ64_N;

                          // outputs to appl. side                
output [63:0]        O_data;
output               O_data_val_lo;
output               O_data_val_hi;
output [7:0]         O_cbe;
output               O_64bit_mode;
output               O_itr_rd_data_avail;
output               O_itr_do_wr_xfer;
output               O_disconnect;
output               O_error;
               
                          // inouts for PCI side
inout [63:0]         IO_AD;
inout [7:0]          IO_CBE_N;
inout                IO_PAR;
inout                IO_PAR64;
inout                IO_PERR_N;
inout                IO_FRAME_N;
inout                IO_IRDY_N;


// wires
     
wire                 fifo_rst;           
wire                 r_incr_itr_rdptr; 
wire [31:0]          itr_rf_in_data_lo;
wire [31:0]          itr_rf_in_data_hi;
wire [7:0]           itr_rf_in_cbe;
wire [31:0]          I_ad_lo;
wire [31:0]          I_ad_hi;
wire [7:0]           I_cbe;  
wire                 data_present; 
wire                 irf_empty;       
wire                 iwf_empty;
wire                 iwf_full;
wire                 irf_full;
wire                 itr_rd_data_avail;
wire                 itr_do_wr_xfer;
wire [7:0]           itr_wf_cbe;
wire [31:0]          itr_wf_data_lo;
wire [31:0]          itr_wf_data_hi;
wire                 io_rd;
wire                 io_wr;
wire                 mem_rd;
wire                 mem_wr;
wire                 dac;
wire                 latched_req32;
wire                 latched_req64;
wire [3:0]           latched_pci_cbe_lo;
wire [3:0]           latched_pci_cbe_hi;
wire [31:0]          latched_addr_lo;
wire [31:0]          latched_addr_hi;
wire                 first_xfer_cycle;
wire                 second_xfer_cycle;
wire                 do_disconnect;
wire                 addr_par;
wire                 addr_par64;
wire                 latched_end_xfer;
wire                 mem_spc_enb;
wire                 io_spc_enb;                   
wire                 frame_N;
wire                 irdy_N;
wire                 trdy_N;
wire                 devsel_N;
wire                 incr_itr_wrptr;
wire                 incr_itr_rdptr;
wire                 oe_par;
wire                 oe_par64;
wire                 oe_AD_lo;
wire                 oe_AD_hi;
wire                 oe_ADDR_lo;
wire                 oe_ADDR_hi;
wire                 incapable64;

wire [3:0]           pci_cbe_lo;
wire [3:0]           pci_cbe_hi;
wire [31:0]          pci_addr_lo;
wire [31:0]          pci_addr_hi;
wire                 prty_err_resp;   
wire [31:0]          in_data_lo;
wire [31:0]          in_data_hi;
wire [3:0]           in_cbe_lo;
wire [3:0]           in_cbe_hi;
wire [3:0]           itr_rf_in_cbe_lo;
wire [3:0]           itr_rf_in_cbe_hi;
wire                 set_detected_prty_err;
wire                 set_prty_err_reported;
wire [3:0]           I_cbe_lo;
wire [3:0]           I_cbe_hi;
wire [31:0]          O_data_lo;
wire [31:0]          O_data_hi;
wire                 set_itr_abort_detected;//new
wire                 set_tgt_abort_detected;//new
wire                 oe_frame;//new
wire                 oe_irdy;//new
wire                 wr_mode;

wire                 disconnect;

assign I_ad_lo = I_ad[31:0];
assign I_ad_hi = I_ad[63:32];

assign O_data_lo = O_data[31:0];
assign O_data_hi = O_data[63:32];

assign I_cbe_lo = I_cbe[3:0];
assign I_cbe_hi = I_cbe[7:4];


  
itr_rw_fifo  mod_itr_rw_fifo
                     (
                     .I_CLK                          (I_CLK),
                     .I_RST_N                        (I_RST_N),
                     .I_app_clk                      (I_app_clk),
                     .I_fifo_reset                   (fifo_rst), 
                     .I_data_val_lo                  (I_data_val_lo),
                     .I_data_val_hi                  (I_data_val_hi),
                     .I_itr_rd_nxt                   (I_itr_rd_nxt),
                     .I_incr_itr_wrptr               (incr_itr_wrptr),
                     .I_incr_itr_rdptr               (r_incr_itr_rdptr),
                     .I_itr_rf_in_data_lo            (itr_rf_in_data_lo),
                     .I_itr_rf_in_data_hi            (itr_rf_in_data_hi),
                     .I_itr_rf_in_cbe                (itr_rf_in_cbe),
                     .I_ad_lo                        (I_ad_lo),
                     .I_ad_hi                        (I_ad_hi),
                     .I_cbe                          (I_cbe),
                     
                     
                     .O_data_present                 (data_present),
                     .O_irf_empty                    (irf_empty),
                     .O_irf_full                     (irf_full),
                     .O_iwf_empty                    (iwf_empty),
                     .O_iwf_full                     (iwf_full),
                     .O_itr_rd_data_avail            (itr_rd_data_avail),
                     .O_irf_cbe                      (O_cbe),
                     .O_iwf_cbe                      (itr_wf_cbe),
                     .O_iwf_data_lo                  (itr_wf_data_lo),
                     .O_iwf_data_hi                  (itr_wf_data_hi),
                     .O_irf_data_lo                  (O_data_lo),
                     .O_irf_data_hi                  (O_data_hi)
                     
                     );             
    

    
itr_cmd_decode  mod_itr_cmd_decode
                   (
                     .I_CLK                          (I_CLK),
                     .I_RST_N                        (I_RST_N),
                     .I_app_clk                      (I_app_clk),
                     .I_cbe_lo                       (I_cbe_lo),
                     .I_cbe_hi                       (I_cbe_hi),
                     .I_itr_end_xfer                 (I_itr_end_xfer),
                     .I_itr_a2c_req32                (I_itr_a2c_req32),                   
                     .I_itr_a2c_req64                (I_itr_a2c_req64),
                     .I_val_addr_lo                  (I_val_addr_lo),
                     .I_val_addr_hi                  (I_val_addr_hi),
                     .I_ad_lo                        (I_ad_lo),
                     .I_ad_hi                        (I_ad_hi),
                     .I_itr_enb                      (I_itr_enb),
                     
                     .O_io_rd                        (io_rd),
                     .O_io_wr                        (io_wr),
                     .O_mem_rd                       (mem_rd),
                     .O_mem_wr                       (mem_wr),
                     .O_dac                          (dac),
                     .O_req32                        (latched_req32),
                     .O_error                        (O_error),
                     .O_req64                        (latched_req64),
                     .O_latched_cbe_lo               (latched_pci_cbe_lo),
                     .O_latched_cbe_hi               (latched_pci_cbe_hi),
                     .O_latched_addr_lo              (latched_addr_lo),
                     .O_latched_addr_hi              (latched_addr_hi)
                  );    
                     

                   
itr_addr_cntrl  mod_itr_addr_cntrl
                   (
                     .I_CLK                            (I_CLK),
                     .I_RST_N                          (I_RST_N),
                     .I_GNT_N                          (I_GNT_N),
                     .I_FRAME_N                        (IO_FRAME_N),
                     .I_IRDY_N                         (IO_IRDY_N),
                     .I_itr_end_xfer                   (I_itr_end_xfer),   
                     .I_latched_addr_lo                (latched_addr_lo),
                     .I_latched_addr_hi                (latched_addr_hi),
                     .I_val_addr_lo                    (I_val_addr_lo),
                     .I_val_addr_hi                    (I_val_addr_hi),
                     .I_latched_cbe_hi                 (latched_pci_cbe_hi),
                     .I_latched_cbe_lo                 (latched_pci_cbe_lo),
                     .I_val_req32                      (latched_req32),
                     .I_val_req64                      (latched_req64),
                     .I_dac                            (dac),
                     
                     .O_first_xfer_cycle               (first_xfer_cycle),
                     .O_second_xfer_cycle              (second_xfer_cycle),
                     .O_do_disconnect                  (do_disconnect), 
                     .O_addr_par                       (addr_par),
                     .O_addr_par64                     (addr_par64),
                     .O_latched_end_xfer               (latched_end_xfer)
                   ); 




itr_engine  mod_itr_engine
                   (
                     .I_CLK                            (I_CLK),
                     .I_RST_N                          (I_RST_N),
                     .I_FRAME_N                        (IO_FRAME_N),//new
                     .I_IRDY_N                         (IO_IRDY_N),//new
                     .I_DEVSEL_N                       (I_DEVSEL_N),
                     .I_TRDY_N                         (I_TRDY_N),
                     .I_STOP_N                         (I_STOP_N),
                     .I_ACK64_N                        (I_ACK64),
                     .I_GNT_N                          (I_GNT_N),
                     .I_retry_disc                     (I_retry_disc),
                     .I_itr_end_xfer                   (I_itr_end_xfer),
                     .I_latched_addr_lo                (latched_addr_lo),
                     .I_latched_addr_hi                (latched_addr_hi),
                     .I_val_addr_lo                    (I_val_addr_lo),
                     .I_val_addr_hi                    (I_val_addr_hi),
                     .I_latched_cbe_hi                 (latched_pci_cbe_hi),
                     .I_latched_cbe_lo                 (latched_pci_cbe_lo),
                     .I_valid_req32                    (latched_req32),
                     .I_valid_req64                    (latched_req64),
                     .I_io_rd                          (io_rd),
                     .I_io_wr                          (io_wr),
                     .I_mem_rd                         (mem_rd),
                     .I_mem_wr                         (mem_wr),
                     .I_dac                            (dac),
                     .I_first_xfer_cycle               (first_xfer_cycle),
                     .I_second_xfer_cycle              (second_xfer_cycle),
                     .I_iwf_full                       (iwf_full),
                     .I_iwf_empty                      (iwf_empty),
                     .I_irf_empty                      (irf_empty),
                     .I_irf_full                       (irf_full),
                     .I_data_present                   (data_present),
                     .I_mem_spc_enb                    (mem_spc_enb),
                     .I_io_spc_enb                     (io_spc_enb),
                     .I_itr_enb                        (I_itr_enb),  
                     .I_do_disconnect                  (do_disconnect),
                                      
                                         
                     .O_disconnect                     (disconnect),
                     .O_REQ_N                          (O_REQ_N),
                     .O_REQ64_N                        (O_REQ64_N),
                     .O_frame_N                        (frame_N),
                     .O_oe_frame                       (oe_frame), 
                     .O_irdy_N                         (irdy_N),
                     .O_oe_irdy                        (oe_irdy),  
                     .O_trdy_N                         (trdy_N),
                     .O_devsel_N                       (devsel_N),
                     .O_incr_iwf_wrptr                 (incr_itr_wrptr),
                     .O_incr_irf_rdptr                 (incr_itr_rdptr),
                     .O_oe_par                         (oe_par),
                     .O_oe_par64                       (oe_par64),
                     .O_oe_AD_lo                       (oe_AD_lo),
                     .O_oe_AD_hi                       (oe_AD_hi),
                     .O_oe_ADDR_lo                     (oe_ADDR_lo),  
                     .O_oe_ADDR_hi                     (oe_ADDR_hi),
                     .O_incapable64                    (incapable64), 
                     .O_pci_cbe_lo                     (pci_cbe_lo),
                     .O_pci_cbe_hi                     (pci_cbe_hi),
                     .O_pci_addr_lo                    (pci_addr_lo),
                     .O_pci_addr_hi                    (pci_addr_hi),
                     .O_set_itr_abort_detected         (set_itr_abort_detected),//new
                     .O_set_tgt_abort_detected         (set_tgt_abort_detected),//new
                     .O_wr_mode                        (wr_mode) //new                    
                   );                                                   
                   

                   
itr_wr_dp  mod_itr_wr_dp
                  (
                     .I_CLK                            (I_CLK),
                     .I_RST_N                          (I_RST_N),
                     .I_irdy_N                         (irdy_N),
                     .I_trdy_N                         (trdy_N),
                     .I_devsel_N                       (devsel_N),
                     .I_itr_wf_data_lo                 (itr_wf_data_lo),
                     .I_itr_wf_data_hi                 (itr_wf_data_hi),
                     .I_itr_wf_cbe_lo                  (itr_wf_cbe_lo),
                     .I_itr_wf_cbe_hi                  (itr_wf_cbe_hi),
                     
                     .O_latched_data_lo                (latched_data_lo),
                     .O_latched_data_hi                (latched_data_hi),
                     .O_latched_cbe_lo                 (latched_cbe_lo),
                     .O_latched_cbe_hi                 (latched_cbe_hi),
                     .O_par                            (par),
                     .O_par64                          (par64)
                     
                   
                  );
                  

           
itr_rd_dp_pchk  mod_itr_rd_dp_pchk
                 (
                     .I_CLK                            (I_CLK),
                     .I_RST_N                          (I_RST_N),

                     .I_PAR                            (IO_PAR),
                     .I_PAR64                          (IO_PAR64),
                     .I_incr_rdptr                     (r_incr_itr_rdptr),
                     .I_prty_err_resp                  (prty_err_resp),

                     .I_in_data_lo                     (in_data_lo),
                     .I_in_data_hi                     (in_data_hi),
                     .I_irdy_N                         (IO_IRDY_N), //changed irdy_N
                     .I_trdy_N                         (I_TRDY_N),  //changed trdy_N
                     .I_devsel_N                       (I_DEVSEL_N),//changed devsel_N                  
                     .I_in_cbe_lo                      (in_cbe_lo),
                     .I_in_cbe_hi                      (in_cbe_hi),
                

                     .O_itr_rf_in_data_lo              (itr_rf_in_data_lo),
                     .O_itr_rf_in_data_hi              (itr_rf_in_data_hi),
                     .O_itr_rf_in_cbe_lo               (itr_rf_in_cbe_lo),
                     .O_itr_rf_in_cbe_hi               (itr_rf_in_cbe_hi),
                     .O_set_detected_prty_err          (set_detected_prty_err),
                     .O_set_prty_err_reported          (set_prty_reported),

                     .IO_PERR_N                        (IO_PERR_N)  // later should change to I_perr.
                     
                 
                 );                                      

itr_misc  mod_itr_misc
                   (
                     .I_CLK                           (I_CLK), 
                     .I_RST_N                         (I_RST_N),
                     .I_DEVSEL_N                      (I_DEVSEL_N),
                     .I_ACK64_N                       (I_ACK64_N),//new
                     
                     .I_appl_clk                      (I_appl_clk), 
                     .I_end_xfer                      (latched_end_xfer),
                     .I_itr_rd_data_avail             (itr_rd_data_avail),
                     .I_itr_rd_nxt                    (I_itr_rd_nxt), 
                     .I_itr_wf_full                   (iwf_full),
                     .I_frame_N                       (frame_N),
                     .I_oe_frame                      (oe_frame), 
                     .I_irdy_N                        (irdy_N),
                     .I_oe_irdy                       (oe_irdy),
                     .I_oe_AD_hi                      (oe_AD_hi),
                     .I_oe_AD_lo                      (oe_AD_lo),
                     .I_oe_ADDR_lo                    (oe_ADDR_lo),  
                     .I_oe_ADDR_hi                    (oe_ADDR_hi),
                     .I_incapable64                   (incapable64), 
                     .I_oe_PAR                        (oe_par),
                     .I_oe_PAR64                      (oe_par64),
                     .I_incr_itr_rdptr                (incr_itr_rdptr),
                     .I_par                           (par),
                     .I_par64                         (par64),
                     .I_pci_addr_lo                   (pci_addr_lo), 
                     .I_pci_addr_hi                   (pci_addr_hi), 
                     .I_pci_cbe_lo                    (pci_cbe_lo), 
                     .I_pci_cbe_hi                    (pci_cbe_hi), 
                     .I_latched_data_lo               (latched_data_lo),
                     .I_latched_data_hi               (latched_data_hi),
                     .I_latched_cbe_lo                (latched_cbe_lo),
                     .I_latched_cbe_hi                (latched_cbe_hi),
                     .I_wr_mode                       (wr_mode), //new
                     .I_addr_par                      (addr_par),//new                     
                     .I_addr_par64                    (addr_par64),//new                     
                     .I_disconnect                    (disconnect),
 
                     .IO_AD                           (IO_AD),
                     .IO_CBE_N                        (IO_CBE_N),
                     .IO_PAR                          (IO_PAR),
                     .IO_PAR64                        (IO_PAR64),
                     .IO_FRAME_N                      (IO_FRAME_N),
                     .IO_IRDY_N                       (IO_IRDY_N),

                     .O_itr_do_wr_xfer                (itr_do_wr_xfer),
                     .O_data_valid_lo                 (O_data_valid_lo),
                     .O_data_valid_hi                 (O_data_valid_hi),
                     .O_r_incr_itr_rdptr              (r_incr_itr_rdptr),
                     .O_fifo_rst                      (fifo_rst),
                     .O_in_data_lo                    (in_data_lo),
                     .O_in_data_hi                    (in_data_hi),
                     .O_in_cbe_lo                     (in_cbe_lo),
                     .O_in_cbe_hi                     (in_cbe_hi),
                     .O_disconnect                    (O_disconnect),
                     .O_64bit_mode                    (O_64bit_mode) 
                    
                   );
endmodule 


 







