/**************************************************************************
		Company     : chiplogic 
		File        : Initiator write data path
		Created on  : 10th june 1999
		modified    : D.A.Naga raju
		Description : This a Initiator write data path for PCI controller
					  with asynchronous reset and synchronous data transfer 
***************************************************************************/
`include "itr_header.v"

module itr_wr_dp (  I_CLK,
					I_RST_N,
					I_irdy_n,
					I_trdy_n,
					I_devsel_n,
					I_itr_wf_data_lo,
					I_itr_wf_data_hi,
					I_itr_wf_cbe_lo,
					I_itr_wf_cbe_hi,
					O_latched_data_lo,
					O_latched_data_hi,
					O_latched_cbe_lo,
					O_latched_cbe_hi,
					O_par,
					O_par64
					);
					
	input     	                        I_CLK;
	input     	                        I_RST_N;
	input		[`DATA_HALF_WIDTH-1:0]  I_itr_wf_data_lo;
	input		[`DATA_HALF_WIDTH-1:0]  I_itr_wf_data_hi;
	input		[`CBE_HALF_WIDTH-1:0]   I_itr_wf_cbe_lo;
	input		[`CBE_HALF_WIDTH-1:0]   I_itr_wf_cbe_hi;
	input      	                        I_devsel_n;
	input      	                        I_irdy_n;
	input      	                        I_trdy_n;
	
	output		[`DATA_HALF_WIDTH-1:0]  O_latched_data_lo;
	output		[`DATA_HALF_WIDTH-1:0]  O_latched_data_hi;
	output      [`CBE_HALF_WIDTH-1:0]   O_latched_cbe_lo;
	output      [`CBE_HALF_WIDTH-1:0]   O_latched_cbe_hi;
	output                              O_par;
	output                              O_par64;
	
	reg			[`DATA_HALF_WIDTH-1:0]  O_latched_data_lo;
	reg			[`DATA_HALF_WIDTH-1:0]  O_latched_data_hi;
	reg         [`CBE_HALF_WIDTH-1:0]   O_latched_cbe_lo;
	reg         [`CBE_HALF_WIDTH-1:0]   O_latched_cbe_hi;
	reg 		                        O_par;
	reg 		                        O_par64;



always @(posedge I_CLK or negedge I_RST_N)        

	begin
		if(~I_RST_N)

			begin      //reset the output
				O_par    <= 1'b0;				
				O_par64  <= 1'b0;
			end

		else if(~I_devsel_n)

			begin     
				O_par    <= ^({I_itr_wf_data_lo,I_itr_wf_cbe_lo}); //((^I_itr_wf_data_lo) ^ (^I_itr_wf_cbe_lo)); 
				O_par64  <= ^({I_itr_wf_data_hi,I_itr_wf_cbe_hi});			
			end		
	end


always @(posedge I_CLK or negedge I_RST_N)        

	begin
		if(~I_RST_N)

			begin      //reset the output
				O_latched_data_lo <= 32'h00000000;
				O_latched_data_hi <= 32'h00000000;
				O_latched_cbe_lo  <= 4'h0;
				O_latched_cbe_hi  <= 4'h0;
			end

		//check for device selection and ready signals
		else if(~I_devsel_n && ~I_irdy_n && ~I_trdy_n)
			
			begin      //data assignment from input to output
				O_latched_data_lo <= I_itr_wf_data_lo;
				O_latched_data_hi <= I_itr_wf_data_hi;
				O_latched_cbe_lo  <= I_itr_wf_cbe_lo;
				O_latched_cbe_hi  <= I_itr_wf_cbe_hi;
			end
	end

endmodule	